/**
Created by Andreas Weggel, https://crazyants.de
**/
window.onload = function () {
    var controller = new AntCounterController();
};
var AntCounterState;
(function (AntCounterState) {
    AntCounterState[AntCounterState["ADDMARKER"] = 0] = "ADDMARKER";
    AntCounterState[AntCounterState["DELETEMARKER"] = 1] = "DELETEMARKER";
})(AntCounterState || (AntCounterState = {}));
var AntCounterController = (function () {
    function AntCounterController() {
        this.state = AntCounterState.ADDMARKER;
        this.markerType = MarkerType.WORKER;
        this.model = undefined;
        this.savedModels = {};
        this.view = new AntCounterView(this);
    }
    AntCounterController.prototype.openFile = function (file) {
        if (file != undefined) {
            var jsonString = localStorage.getItem(file.size.toString());
            if (jsonString === null) {
                this.model = new AntCounterModel();
                this.model.setFile(file);
                this.model.save();
            }
            else {
                this.model = new AntCounterModel();
                var jsonObject = JSON.parse(jsonString);
                this.model.loadFromJson(jsonObject);
                this.model.setFile(file);
            }
            this.view.render(this.model);
        }
    };
    AntCounterController.prototype.clickAction = function (x, y, radius) {
        if (this.state === AntCounterState.ADDMARKER) {
            this.addMarker(x, y, this.markerType);
        }
        else {
            this.deleteMarker(x, y, radius);
        }
    };
    AntCounterController.prototype.addMarker = function (x, y, type) {
        var marker = new MarkerModel();
        var scale = this.model.getScale();
        marker.setX(x / scale);
        marker.setY(y / scale);
        marker.setType(type);
        this.model.addMarker(marker);
        this.view.render(this.model);
        this.model.save();
    };
    AntCounterController.prototype.deleteMarker = function (x, y, radius) {
        var scale = this.model.getScale();
        this.model.deleteMarker(x / scale, y / scale, radius + 10);
        this.view.render(this.model);
        this.model.save();
    };
    AntCounterController.prototype.setScale = function (scale) {
        if (this.model !== undefined) {
            this.model.setScale(scale);
            this.view.render(this.model);
            this.model.save();
        }
    };
    AntCounterController.prototype.setMarkerScale = function (markerScale) {
        if (this.model !== undefined) {
            this.model.setMarkerScale(markerScale);
            this.view.render(this.model);
            this.model.save();
        }
    };
    AntCounterController.prototype.setState = function (state) {
        this.state = state;
    };
    AntCounterController.prototype.setMarkerType = function (markerType) {
        this.markerType = markerType;
    };
    return AntCounterController;
}());
var AntCounterModel = (function () {
    function AntCounterModel() {
        this.markers = [];
        this.scale = 0.5;
        this.markerScale = 1;
        this.markerCount = {};
        this.markerCount[MarkerType.WORKER] = 0;
        this.markerCount[MarkerType.QUEEN] = 0;
        this.markerCount[MarkerType.EGG] = 0;
        this.markerCount[MarkerType.LARVAE] = 0;
        this.markerCount[MarkerType.PUPAE] = 0;
    }
    AntCounterModel.prototype.getMarkers = function () {
        return this.markers;
    };
    AntCounterModel.prototype.getFile = function () {
        return this.file;
    };
    AntCounterModel.prototype.setFile = function (file) {
        this.file = file;
    };
    AntCounterModel.prototype.addMarker = function (marker) {
        this.markers.push(marker);
        this.markerCount[marker.getType()]++;
    };
    AntCounterModel.prototype.deleteMarker = function (x, y, radius) {
        var foundMarkerIndex = this.findMarker(x, y, radius * this.markerScale);
        var foundMarker = this.markers[foundMarkerIndex];
        if (foundMarkerIndex !== undefined) {
            this.markers.splice(foundMarkerIndex, 1);
        }
        this.markerCount[foundMarker.getType()]--;
    };
    AntCounterModel.prototype.findMarker = function (x, y, radius) {
        var returnValue = undefined;
        for (var i = this.markers.length - 1; i >= 0; i--) {
            var currentMarker = this.markers[i];
            var markerX = currentMarker.getX();
            var markerY = currentMarker.getY();
            var distance = Math.sqrt(Math.pow((markerX - x), 2) + Math.pow((markerY - y), 2));
            if (distance <= radius) {
                returnValue = i;
                break;
            }
        }
        return returnValue;
    };
    AntCounterModel.prototype.getMarkerCount = function (type) {
        return this.markerCount[type];
    };
    AntCounterModel.prototype.getScale = function () {
        return this.scale;
    };
    AntCounterModel.prototype.setScale = function (scale) {
        this.scale = scale;
    };
    AntCounterModel.prototype.getMarkerScale = function () {
        return this.markerScale;
    };
    AntCounterModel.prototype.setMarkerScale = function (markerScale) {
        this.markerScale = markerScale;
    };
    AntCounterModel.prototype.save = function () {
        var jsonString = JSON.stringify(this, function (key, value) {
            if (key === 'file') {
                return undefined;
            }
            return value;
        });
        localStorage.setItem(this.file.size.toString(), jsonString);
    };
    AntCounterModel.prototype.loadFromJson = function (jsonObject) {
        this.scale = jsonObject.scale;
        this.markerCount = jsonObject.markerCount;
        this.markerScale = jsonObject.markerScale;
        var object = jsonObject.markers;
        for (var i = 0; i < object.length; i++) {
            var marker = new MarkerModel();
            marker.setType(object[i].markerType);
            marker.setX(object[i].x);
            marker.setY(object[i].y);
            this.addMarker(marker);
        }
    };
    return AntCounterModel;
}());
var MarkerType;
(function (MarkerType) {
    MarkerType[MarkerType["WORKER"] = 0] = "WORKER";
    MarkerType[MarkerType["QUEEN"] = 1] = "QUEEN";
    MarkerType[MarkerType["EGG"] = 2] = "EGG";
    MarkerType[MarkerType["LARVAE"] = 3] = "LARVAE";
    MarkerType[MarkerType["PUPAE"] = 4] = "PUPAE";
})(MarkerType || (MarkerType = {}));
var MarkerModel = (function () {
    function MarkerModel() {
    }
    MarkerModel.prototype.getType = function () {
        return this.markerType;
    };
    MarkerModel.prototype.setType = function (markerType) {
        this.markerType = markerType;
    };
    MarkerModel.prototype.getX = function () {
        return this.x;
    };
    MarkerModel.prototype.setX = function (x) {
        this.x = x;
    };
    MarkerModel.prototype.getY = function () {
        return this.y;
    };
    MarkerModel.prototype.setY = function (y) {
        this.y = y;
    };
    return MarkerModel;
}());
var AntCounterView = (function () {
    function AntCounterView(antCounterController) {
        var _this = this;
        this.fillStyles = {};
        this.markerRadius = 5;
        this.markerLineWidth = 3;
        this.loadedFile = undefined;
        this.devicePixelRation = window.devicePixelRatio;
        this.initFillStyles();
        this.antCounterController = antCounterController;
        this.workerInfoCanvas = document.getElementById('workerInfoCanvas');
        this.renderInfoCanvas(this.workerInfoCanvas, MarkerType.WORKER);
        this.workerCount = document.getElementById('worker-count');
        this.queenInfoCanvas = document.getElementById('queenInfoCanvas');
        this.renderInfoCanvas(this.queenInfoCanvas, MarkerType.QUEEN);
        this.queenCount = document.getElementById('queen-count');
        this.eggInfoCanvas = document.getElementById('eggInfoCanvas');
        this.renderInfoCanvas(this.eggInfoCanvas, MarkerType.EGG);
        this.eggCount = document.getElementById('egg-count');
        this.larvaeInfoCanvas = document.getElementById('larvaeInfoCanvas');
        this.renderInfoCanvas(this.larvaeInfoCanvas, MarkerType.LARVAE);
        this.larvaeCount = document.getElementById('larvae-count');
        this.pupaeInfoCanvas = document.getElementById('pupaeInfoCanvas');
        this.renderInfoCanvas(this.pupaeInfoCanvas, MarkerType.PUPAE);
        this.pupaeCount = document.getElementById('pupae-count');
        this.fileChooser = document.getElementById("fileChooser");
        this.scaleRange = document.getElementById('scaleRange');
        this.markerSizeRange = document.getElementById('markerSizeRange');
        this.initRadioButtons();
        this.canvas = document.getElementById('canvas');
        this.canvas.addEventListener('click', function (evt) {
            var boundingRect = _this.canvas.getBoundingClientRect();
            var top = boundingRect.top;
            var left = boundingRect.left;
            var x = evt.clientX - left;
            var y = evt.clientY - top;
            _this.antCounterController.clickAction(x, y, _this.markerRadius + _this.markerLineWidth);
        });
        this.imgElement = document.createElement('img');
        this.context = this.canvas.getContext('2d');
        this.fileChooser.addEventListener('change', function (evt) {
            var file = _this.fileChooser.files[0];
            _this.antCounterController.openFile(file);
        });
        this.scaleRange.addEventListener('input', function (evt) {
            _this.antCounterController.setScale(+_this.scaleRange.value);
        });
        this.markerSizeRange.addEventListener('input', function (evt) {
            var markerSize = parseFloat(_this.markerSizeRange.value);
            _this.antCounterController.setMarkerScale(markerSize);
        });
    }
    AntCounterView.prototype.renderInfoCanvas = function (canvas, markerType) {
        var context = canvas.getContext('2d');
        var width = canvas.width;
        var height = canvas.height;
        var style = this.fillStyles[markerType];
        var marker = new MarkerModel();
        marker.setX(width / 2.0);
        marker.setY(height / 2.0);
        marker.setType(markerType);
        this.drawMarker(context, marker, 1, 1);
    };
    AntCounterView.prototype.initRadioButtons = function () {
        var _this = this;
        this.countRadioButton = document.getElementById("countRadioButton");
        this.addMarkerSpan = document.getElementById('addMarkerSpan');
        this.markerType = document.getElementById("markerType");
        this.deleteMarkerSpan = document.getElementById('deleteMarkerSpan');
        this.deleteRadioButton = document.getElementById("deleteRadioButton");
        this.addMarkerSpan.addEventListener('click', function (evt) {
            _this.countRadioButton.checked = true;
            _this.markerType.disabled = false;
            _this.antCounterController.setState(parseInt(_this.countRadioButton.value));
        });
        this.deleteMarkerSpan.addEventListener('click', function (evt) {
            _this.deleteRadioButton.checked = true;
            _this.markerType.disabled = true;
            _this.antCounterController.setState(parseInt(_this.deleteRadioButton.value));
        });
        this.markerType.addEventListener('change', function (evt) {
            _this.antCounterController.setMarkerType(parseInt(_this.markerType.value));
        });
    };
    AntCounterView.prototype.initFillStyles = function () {
        this.fillStyles[MarkerType.WORKER] = 'blue';
        this.fillStyles[MarkerType.QUEEN] = 'red';
        this.fillStyles[MarkerType.EGG] = 'green';
        this.fillStyles[MarkerType.LARVAE] = 'orange';
        this.fillStyles[MarkerType.PUPAE] = 'sienna';
    };
    AntCounterView.prototype.render = function (model) {
        this.renderInfo(model);
        this.renderRanges(model);
        this.renderCanvas(model);
    };
    AntCounterView.prototype.renderInfo = function (model) {
        var markers = model.getMarkers();
        this.workerCount.innerHTML = model.getMarkerCount(MarkerType.WORKER).toString();
        this.queenCount.innerHTML = model.getMarkerCount(MarkerType.QUEEN).toString();
        this.eggCount.innerHTML = model.getMarkerCount(MarkerType.EGG).toString();
        this.larvaeCount.innerHTML = model.getMarkerCount(MarkerType.LARVAE).toString();
        this.pupaeCount.innerHTML = model.getMarkerCount(MarkerType.PUPAE).toString();
    };
    AntCounterView.prototype.renderRanges = function (model) {
        var scale = model.getScale();
        var markerSize = model.getMarkerScale();
        this.markerSizeRange.value = markerSize.toString();
        this.scaleRange.value = scale.toString();
    };
    AntCounterView.prototype.renderCanvas = function (model) {
        var _this = this;
        var modelFile = model.getFile();
        if (modelFile !== this.loadedFile) {
            this.loadedFile = modelFile;
            var imgUrl = window.URL.createObjectURL(model.getFile());
            this.imgElement.src = imgUrl;
            this.imgElement.onload = function () {
                _this.draw(model);
                window.URL.revokeObjectURL(imgUrl);
            };
        }
        else {
            this.draw(model);
        }
    };
    AntCounterView.prototype.draw = function (model) {
        var scale = model.getScale();
        var markerScale = model.getMarkerScale();
        var width = this.imgElement.width;
        var height = this.imgElement.height;
        width = width * scale;
        height = height * scale;
        this.canvas.width = width;
        this.canvas.height = height;
        this.context.clearRect(0, 0, width, height);
        this.context.drawImage(this.imgElement, 0, 0, width, height);
        var markers = model.getMarkers();
        var scale = model.getScale();
        this.drawMarkers(markers, scale, markerScale);
    };
    AntCounterView.prototype.drawMarkers = function (markers, scale, markerScale) {
        for (var i = 0; i < markers.length; i++) {
            var marker = markers[i];
            this.drawMarker(this.context, marker, scale, markerScale);
        }
    };
    AntCounterView.prototype.drawMarker = function (context, marker, scale, markerScale) {
        context.beginPath();
        var x = marker.getX();
        var y = marker.getY();
        x = x * scale;
        y = y * scale;
        var radius = this.markerRadius;
        var lineWidth = this.markerLineWidth;
        radius *= markerScale;
        lineWidth *= markerScale;
        context.lineWidth = lineWidth;
        context.strokeStyle = 'white';
        context.arc(x, y, radius, 0, Math.PI * 2);
        context.stroke();
        context.fillStyle = this.fillStyles[marker.getType()];
        context.fill();
    };
    return AntCounterView;
}());
//# sourceMappingURL=antcounter.js.map